@@ -8,15 +8,15 @@
 #include "resultmodel.h"
 
 // Qt
-#include <QDebug>
-#include <QDateTime>
 #include <QCoreApplication>
+#include <QDateTime>
+#include <QDebug>
 #include <QFile>
+#include <QTimer>
 
-// STL and Boost
+// STL
 #include <functional>
-#include <boost/range/algorithm/find_if.hpp>
-#include <boost/range/algorithm/count_if.hpp>
+#include <thread>
 
 // KDE
 #include <KSharedConfig>
@@ -65,7 +65,7 @@ public:
     enum Fetch {
         FetchReset,   // Remove old data and reload
         FetchReload,  // Update all data
-        FetchMore     // Load more data if there is any
+        FetchMore,     // Load more data if there is any
     };
 
     class Cache { //_
@@ -124,7 +124,9 @@ public:
             QStringList linkedItems;
 
             for (const ResultSet::Result &item : qAsConst(m_items)) {
-                if (item.linkStatus() == ResultSet::Result::NotLinked) break;
+                if (item.linkStatus() == ResultSet::Result::NotLinked) {
+                    break;
+                }
                 linkedItems << item.resource();
             }
 
@@ -263,10 +265,10 @@ public:
         inline FindCacheResult find(const QString &resource)
         {
             using namespace kamd::utils::member_matcher;
-            using boost::find_if;
 
+            // Non-const iterator because the result is constructed from it
             return FindCacheResult(
-                this, find_if(m_items, member(&ResultSet::Result::resource)
+                this, std::find_if(m_items.begin(), m_items.end(), member(&ResultSet::Result::resource)
                                            == resource));
         }
 
@@ -274,7 +276,7 @@ public:
         inline FindCacheResult lowerBoundWithSkippedResource(Predicate &&lessThanPredicate)
         {
             using namespace kamd::utils::member_matcher;
-            const int count = boost::count_if(m_items,
+            const int count = std::count_if(m_items.cbegin(), m_items.cend(),
                     [&] (const ResultSet::Result &result) {
                         return lessThanPredicate(result, _);
                     });
@@ -327,7 +329,9 @@ public:
 
         inline void clear()
         {
-            if (m_items.size() == 0) return;
+            if (m_items.size() == 0) {
+                return;
+            }
 
             d->q->beginRemoveRows(QModelIndex(), 0, m_items.size() - 1);
             m_items.clear();
@@ -391,7 +395,9 @@ public:
             // in the case where somebody called this with too much new items.
             const int maxToReplace = m_countLimit - from;
 
-            if (maxToReplace <= 0) return;
+            if (maxToReplace <= 0) {
+                return;
+            }
 
             const auto newItemsEnd =
                 newItems.size() <= maxToReplace ? newItems.cend() :
@@ -489,11 +495,24 @@ public:
 
             // Check whether we got an item representing a non-existent file,
             // if so, schedule its removal from the database
-            for (const auto &item: newItems) {
-                if (item.resource().startsWith(QLatin1Char('/')) && !QFile(item.resource()).exists()) {
-                    d->q->forgetResource(item.resource());
+            // we want to do this async so that we don't block
+            std::thread([=] {
+                QList<QString> missingResources;
+                for (const auto &item: newItems) {
+                    // QFile.exists() can be incredibly slow (eg. if resource is on remote filesystem)
+                    if (item.resource().startsWith(QLatin1Char('/')) && !QFile(item.resource()).exists()) {
+                        missingResources << item.resource();
+                    }
                 }
-            }
+
+                if (missingResources.empty()) {
+                    return;
+                }
+
+                QTimer::singleShot(0, this->d->q, [=] {
+                    d->q->forgetResources(missingResources);
+                });
+            }).detach();
         }
         //^
 
@@ -504,7 +523,9 @@ public:
 
         inline void trim(int limit)
         {
-            if (m_items.size() <= limit) return;
+            if (m_items.size() <= limit) {
+                return;
+            }
 
             // Example:
             //   limit is 5,
@@ -524,7 +545,7 @@ public:
 
         enum Ordering {
             PartialOrdering,
-            FullOrdering
+            FullOrdering,
         };
 
         FixedItemsLessThan(Ordering ordering,
@@ -544,11 +565,13 @@ public:
             const bool hasLeft  = indexLeft != -1;
             const bool hasRight = indexRight != -1;
 
+            /* clang-format off */
             return
                 ( hasLeft && !hasRight) ? true :
                 (!hasLeft &&  hasRight) ? false :
                 ( hasLeft &&  hasRight) ? indexLeft < indexRight :
                 (ordering == PartialOrdering ? false : leftResource < rightResource);
+            /* clang-format on */
         }
 
         template <typename T>
@@ -586,6 +609,7 @@ public:
         const auto lastUpdate  = result.lastUpdate();
         const auto linkStatus  = result.linkStatus();
 
+        /* clang-format off */
         #define FIXED_ITEMS_LESS_THAN FixedItemsLessThan(FixedItemsLessThan::PartialOrdering, cache, resource)
         #define ORDER_BY(Field) member(&ResultSet::Result::Field) > Field
         #define ORDER_BY_FULL(Field)                                           \
@@ -611,6 +635,8 @@ public:
         #undef ORDER_BY_FULL
         #undef FIXED_ITEMS_LESS_THAN
 
+        /* clang-format on */
+
         return destination;
     }
 
@@ -704,7 +730,9 @@ public:
             count = query.limit() - from;
         }
 
-        if (count <= 0) return;
+        if (count <= 0) {
+            return;
+        }
 
         // In order to see whether there are more results, we need to pass
         // the count increased by one
@@ -740,10 +768,12 @@ public:
             // and loading all from scratch
             cache.clear();
 
+            /* clang-format off */
             const QString activityTag =
                 query.activities().contains(CURRENT_ACTIVITY_TAG)
                     ? (QStringLiteral("-ForActivity-") + activities.currentActivity())
                     : QStringLiteral("-ForAllActivities");
+            /* clang-format on */
 
             cache.loadOrderingConfig(activityTag);
 
@@ -782,11 +812,13 @@ public:
         // we already have it in the cache
         const auto result = cache.find(resource);
 
+        /* clang-format off */
         ResultSet::Result::LinkStatus linkStatus
             = result ? result->linkStatus()
             : query.selection() != Terms::UsedResources ? ResultSet::Result::Unknown
             : query.selection() != Terms::LinkedResources ? ResultSet::Result::Linked
             : ResultSet::Result::NotLinked;
+        /* clang-format on */
 
         if (result) {
             // We are only updating a result we already had,
@@ -836,7 +868,9 @@ public:
     {
         const auto result = cache.find(resource);
 
-        if (!result) return;
+        if (!result) {
+            return;
+        }
 
         if (query.selection() == Terms::UsedResources
             || result->linkStatus() != ResultSet::Result::Linked) {
@@ -855,7 +889,9 @@ public:
     {
         const auto result = cache.find(resource);
 
-        if (!result) return;
+        if (!result) {
+            return;
+        }
 
         if (query.selection() == Terms::LinkedResources) {
             removeResult(result);
@@ -877,8 +913,11 @@ public:
     //_ Title and mimetype functions
     void fillTitleAndMimetype(ResultSet::Result &result)
     {
-        if (!database) return;
+        if (!database) {
+            return;
+        }
 
+        /* clang-format off */
         auto query = database->execQuery(
                 QStringLiteral("SELECT "
                 "title, mimetype "
@@ -887,6 +926,7 @@ public:
                 "WHERE "
                 "targettedResource = '") + result.resource() + QStringLiteral("'")
                 );
+        /* clang-format on */
 
         // Only one item at most
         for (const auto &item: query) {
@@ -899,7 +939,9 @@ public:
     {
         const auto result = cache.find(resource);
 
-        if (!result) return;
+        if (!result) {
+            return;
+        }
 
         result->setTitle(title);
 
@@ -912,7 +954,9 @@ public:
 
         const auto result = cache.find(resource);
 
-        if (!result) return;
+        if (!result) {
+            return;
+        }
 
         result->setMimetype(mimetype);
 
@@ -982,6 +1026,7 @@ QVariant ResultModel::data(const QModelIndex &item, int role) const
 
     const auto &result = d->cache[row];
 
+    /* clang-format off */
     return role == Qt::DisplayRole ? QString(
                result.title() + QStringLiteral(" ") +
                result.resource() + QStringLiteral(" - ") +
@@ -998,6 +1043,7 @@ QVariant ResultModel::data(const QModelIndex &item, int role) const
          : role == MimeType             ? result.mimetype()
          : QVariant()
          ;
+    /* clang-format on */
 }
 
 QVariant ResultModel::headerData(int section, Qt::Orientation orientation,
@@ -1016,7 +1062,9 @@ int ResultModel::rowCount(const QModelIndex &parent) const
 
 void ResultModel::fetchMore(const QModelIndex &parent)
 {
-    if (parent.isValid()) return;
+    if (parent.isValid()) {
+        return;
+    }
     d->fetch(ResultModelPrivate::FetchMore);
 }
 
@@ -1027,33 +1075,46 @@ bool ResultModel::canFetchMore(const QModelIndex &parent) const
          : d->hasMore;
 }
 
-void ResultModel::forgetResource(const QString &resource)
+void ResultModel::forgetResources(const QList<QString> &resources)
 {
     const auto lstActivities = d->query.activities();
     for (const QString &activity : lstActivities) {
         const auto lstAgents = d->query.agents();
         for (const QString &agent : lstAgents) {
-            Stats::forgetResource(
-                    activity,
-                    agent == CURRENT_AGENT_TAG ?
-                        QCoreApplication::applicationName() : agent,
-                    resource);
+            for (const QString &resource : resources) {
+                /* clang-format off */
+                Stats::forgetResource(
+                        activity,
+                        agent == CURRENT_AGENT_TAG ?
+                            QCoreApplication::applicationName() : agent,
+                        resource);
+                /* clang-format on */
+            }
         }
     }
 }
 
+void ResultModel::forgetResource(const QString &resource)
+{
+    ResultModel::forgetResources({ resource });
+}
+
 void ResultModel::forgetResource(int row)
 {
-    if (row >= d->cache.size()) return;
+    if (row >= d->cache.size()) {
+        return;
+    }
     const auto lstActivities = d->query.activities();
     for (const QString &activity : lstActivities) {
         const auto lstAgents = d->query.agents();
         for (const QString &agent : lstAgents) {
+            /* clang-format off */
             Stats::forgetResource(
                     activity,
                     agent == CURRENT_AGENT_TAG ?
                         QCoreApplication::applicationName() : agent,
                     d->cache[row].resource());
+            /* clang-format on */
         }
     }
 }
